@@ -40,9 +40,9 @@ DataType <- R6Class("DataType",
     fields = function() {
       DataType__fields(self)
     },
-    export_to_c = function(ptr) ExportType(self, ptr)
+    export_to_c = function(ptr) ExportType(self, ptr),
+    code = function() call("stop", paste0("Unsupported type: <", self$ToString(), ">."))
   ),
-
   active = list(
     id = function() DataType__id(self),
     name = function() DataType__name(self),
@@ -93,6 +93,9 @@ type.ArrowDatum <- function(x) x$type
 #' @name FixedWidthType
 FixedWidthType <- R6Class("FixedWidthType",
   inherit = DataType,
+  public = list(
+    code = function() call(tolower(self$name))
+  ),
   active = list(
     bit_width = function() FixedWidthType__bit_width(self)
   )
@@ -108,17 +111,48 @@ UInt32 <- R6Class("UInt32", inherit = FixedWidthType)
 UInt64 <- R6Class("UInt64", inherit = FixedWidthType)
 Float16 <- R6Class("Float16", inherit = FixedWidthType)
 Float32 <- R6Class("Float32", inherit = FixedWidthType)
-Float64 <- R6Class("Float64", inherit = FixedWidthType)
+Float64 <- R6Class("Float64",
+  inherit = FixedWidthType,
+  public = list(
+    code = function() call("float64")
+  )
+)
 Boolean <- R6Class("Boolean", inherit = FixedWidthType)
-Utf8 <- R6Class("Utf8", inherit = DataType)
-LargeUtf8 <- R6Class("LargeUtf8", inherit = DataType)
-Binary <- R6Class("Binary", inherit = DataType)
-FixedSizeBinary <- R6Class("FixedSizeBinary", inherit = FixedWidthType)
-LargeBinary <- R6Class("LargeBinary", inherit = DataType)
+Utf8 <- R6Class("Utf8",
+  inherit = DataType,
+  public = list(
+    code = function() call("utf8")
+  )
+)
+LargeUtf8 <- R6Class("LargeUtf8",
+  inherit = DataType,
+  public = list(
+    code = function() call("large_utf8")
+  )
+)
+Binary <- R6Class("Binary",
+  inherit = DataType,
+  public = list(
+    code = function() call("binary")
+  )
+)
+LargeBinary <- R6Class("LargeBinary",
+  inherit = DataType, public = list(
+    code = function() call("large_binary")
+  )
+)
+FixedSizeBinary <- R6Class("FixedSizeBinary",
+  inherit = FixedWidthType,
+  public = list(
+    byte_width = function() FixedSizeBinary__byte_width(self),
+    code = function() call2("fixed_size_binary", byte_width = self$byte_width())
+  )
+)
 
 DateType <- R6Class("DateType",
   inherit = FixedWidthType,
   public = list(
+    code = function() call2(tolower(self$name)),
     unit = function() DateType__unit(self)
   )
 )
@@ -131,15 +165,60 @@ TimeType <- R6Class("TimeType",
     unit = function() TimeType__unit(self)
   )
 )
-Time32 <- R6Class("Time32", inherit = TimeType)
-Time64 <- R6Class("Time64", inherit = TimeType)
+Time32 <- R6Class("Time32",
+  inherit = TimeType,
+  public = list(
+    code = function() {
+      unit <- if (self$unit() == TimeUnit$MILLI) {
+        "ms"
+      } else {
+        "s"
+      }
+      call2("time32", unit = unit)
+    }
+  )
+)
+Time64 <- R6Class("Time64",
+  inherit = TimeType,
+  public = list(
+    code = function() {
+      unit <- if (self$unit() == TimeUnit$NANO) {
+        "ns"
+      } else {
+        "us"
+      }
+      call2("time64", unit = unit)
+    }
+  )
+)
 
-Null <- R6Class("Null", inherit = DataType)
+DurationType <- R6Class("DurationType",
+  inherit = FixedWidthType,
+  public = list(
+    unit = function() DurationType__unit(self)
+  )
+)
+
+Null <- R6Class("Null",
+  inherit = DataType,
+  public = list(
+    code = function() call("null")
+  )
+)
 
 Timestamp <- R6Class("Timestamp",
   inherit = FixedWidthType,
   public = list(
-    timezone = function()  TimestampType__timezone(self),
+    code = function() {
+      unit <- c("s", "ms", "us", "ns")[self$unit() + 1L]
+      tz <- self$timezone()
+      if (identical(tz, "")) {
+        call2("timestamp", unit = unit)
+      } else {
+        call2("timestamp", unit = unit, timezone = tz)
+      }
+    },
+    timezone = function() TimestampType__timezone(self),
     unit = function() TimestampType__unit(self)
   )
 )
@@ -147,12 +226,18 @@ Timestamp <- R6Class("Timestamp",
 DecimalType <- R6Class("DecimalType",
   inherit = FixedWidthType,
   public = list(
+    code = function() {
+      call2("decimal", precision = self$precision(), scale = self$scale())
+    },
     precision = function() DecimalType__precision(self),
     scale = function() DecimalType__scale(self)
   )
 )
+
 Decimal128Type <- R6Class("Decimal128Type", inherit = DecimalType)
 
+Decimal256Type <- R6Class("Decimal256Type", inherit = DecimalType)
+
 NestedType <- R6Class("NestedType", inherit = DataType)
 
 #' Apache Arrow data types
@@ -182,15 +267,50 @@ NestedType <- R6Class("NestedType", inherit = DataType)
 #' `bit64::integer64` object) by setting `options(arrow.int64_downcast =
 #' FALSE)`.
 #'
+#' `decimal128()` creates a `Decimal128Type`. Arrow decimals are fixed-point
+#' decimal numbers encoded as a scalar integer. The `precision` is the number of
+#' significant digits that the decimal type can represent; the `scale` is the
+#' number of digits after the decimal point. For example, the number 1234.567
+#' has a precision of 7 and a scale of 3. Note that `scale` can be negative.
+#'
+#' As an example, `decimal128(7, 3)` can exactly represent the numbers 1234.567 and
+#' -1234.567 (encoded internally as the 128-bit integers 1234567 and -1234567,
+#' respectively), but neither 12345.67 nor 123.4567.
+#'
+#' `decimal128(5, -3)` can exactly represent the number 12345000 (encoded
+#' internally as the 128-bit integer 12345), but neither 123450000 nor 1234500.
+#' The `scale` can be thought of as an argument that controls rounding. When
+#' negative, `scale` causes the number to be expressed using scientific notation
+#' and power of 10.
+#'
+#' `decimal256()` creates a `Decimal256Type`, which allows for higher maximum
+#' precision. For most use cases, the maximum precision offered by `Decimal128Type`
+#' is sufficient, and it will result in a more compact and more efficient encoding.
+#'
+#' #' `decimal()` creates either a `Decimal128Type` or a `Decimal256Type`
+#' depending on the value for `precision`. If `precision` is greater than 38 a
+#' `Decimal256Type` is returned, otherwise a `Decimal128Type`.
+#'
+#' Use `decimal128()` or `decimal256()` as the names are more informative than
+#' `decimal()`.
+#'
 #' @param unit For time/timestamp types, the time unit. `time32()` can take
 #' either "s" or "ms", while `time64()` can be "us" or "ns". `timestamp()` can
 #' take any of those four values.
 #' @param timezone For `timestamp()`, an optional time zone string.
 #' @param byte_width byte width for `FixedSizeBinary` type.
 #' @param list_size list size for `FixedSizeList` type.
-#' @param precision For `decimal()`, precision
-#' @param scale For `decimal()`, scale
+#' @param precision For `decimal()`, `decimal128()`, and `decimal256()` the
+#'    number of significant digits the arrow `decimal` type can represent. The
+#'    maximum precision for `decimal128()` is 38 significant digits, while for
+#'    `decimal256()` it is 76 digits. `decimal()` will use it to choose which
+#'    type of decimal to return.
+#' @param scale For `decimal()`, `decimal128()`, and `decimal256()` the number
+#'    of digits after the decimal point. It can be negative.
 #' @param type For `list_of()`, a data type to make a list-of-type
+#' @param key_type,item_type For `MapType`, the key and item types.
+#' @param .keys_sorted Use `TRUE` to assert that keys of a `MapType` are
+#'   sorted.
 #' @param ... For `struct()`, a named list of types to define the struct columns
 #'
 #' @name data-type
@@ -312,6 +432,13 @@ valid_time64_units <- c(
   "us" = TimeUnit$MICRO
 )
 
+valid_duration_units <- c(
+  "s" = TimeUnit$SECOND,
+  "ms" = TimeUnit$MILLI,
+  "us" = TimeUnit$MICRO,
+  "ns" = TimeUnit$NANO
+)
+
 make_valid_time_unit <- function(unit, valid_units) {
   if (is.character(unit)) {
     unit <- valid_units[match.arg(unit, choices = names(valid_units))]
@@ -338,6 +465,16 @@ time64 <- function(unit = c("ns", "us")) {
   Time64__initialize(unit)
 }
 
+#' @rdname data-type
+#' @export
+duration <- function(unit = c("s", "ms", "us", "ns")) {
+  if (is.character(unit)) {
+    unit <- match.arg(unit)
+  }
+  unit <- make_valid_time_unit(unit, valid_duration_units)
+  Duration__initialize(unit)
+}
+
 #' @rdname data-type
 #' @export
 null <- function() Null__initialize()
@@ -356,22 +493,58 @@ timestamp <- function(unit = c("s", "ms", "us", "ns"), timezone = "") {
 #' @rdname data-type
 #' @export
 decimal <- function(precision, scale) {
+  args <- check_decimal_args(precision, scale)
+
+  if (args$precision > 38) {
+    decimal256(args$precision, args$scale)
+  } else {
+    decimal128(args$precision, args$scale)
+  }
+}
+
+#' @rdname data-type
+#' @export
+decimal128 <- function(precision, scale) {
+  args <- check_decimal_args(precision, scale)
+  Decimal128Type__initialize(args$precision, args$scale)
+}
+
+#' @rdname data-type
+#' @export
+decimal256 <- function(precision, scale) {
+  args <- check_decimal_args(precision, scale)
+  Decimal256Type__initialize(args$precision, args$scale)
+}
+
+check_decimal_args <- function(precision, scale) {
   if (is.numeric(precision)) {
-    precision <- as.integer(precision)
+    precision <- vec_cast(precision, to = integer())
+    vctrs::vec_assert(precision, size = 1L)
   } else {
-    stop('"precision" must be an integer', call. = FALSE)
+    stop("`precision` must be an integer", call. = FALSE)
   }
+
   if (is.numeric(scale)) {
-    scale <- as.integer(scale)
+    scale <- vec_cast(scale, to = integer())
+    vctrs::vec_assert(scale, size = 1L)
   } else {
-    stop('"scale" must be an integer', call. = FALSE)
+    stop("`scale` must be an integer", call. = FALSE)
   }
-  Decimal128Type__initialize(precision, scale)
+
+  list(precision = precision, scale = scale)
 }
 
 StructType <- R6Class("StructType",
   inherit = NestedType,
   public = list(
+    code = function() {
+      field_names <- StructType__field_names(self)
+      codes <- map(field_names, function(name) {
+        self$GetFieldByName(name)$type$code()
+      })
+      codes <- set_names(codes, field_names)
+      call2("struct", !!!codes)
+    },
     GetFieldByName = function(name) StructType__GetFieldByName(self, name),
     GetFieldIndex = function(name) StructType__GetFieldIndex(self, name)
   )
@@ -384,6 +557,11 @@ struct <- StructType$create
 
 ListType <- R6Class("ListType",
   inherit = NestedType,
+  public = list(
+    code = function() {
+      call("list_of", self$value_type$code())
+    }
+  ),
   active = list(
     value_field = function() ListType__value_field(self),
     value_type = function() ListType__value_type(self)
@@ -396,6 +574,11 @@ list_of <- function(type) list__(type)
 
 LargeListType <- R6Class("LargeListType",
   inherit = NestedType,
+  public = list(
+    code = function() {
+      call2("large_list_of", self$value_type$code())
+    }
+  ),
   active = list(
     value_field = function() LargeListType__value_field(self),
     value_type = function() LargeListType__value_type(self)
@@ -410,6 +593,11 @@ large_list_of <- function(type) large_list__(type)
 #' @export
 FixedSizeListType <- R6Class("FixedSizeListType",
   inherit = NestedType,
+  public = list(
+    code = function() {
+      call2("fixed_size_list_of",  self$value_type$code(), list_size = self$list_size)
+    }
+  ),
   active = list(
     value_field = function() FixedSizeListType__value_field(self),
     value_type = function() FixedSizeListType__value_type(self),
@@ -421,6 +609,23 @@ FixedSizeListType <- R6Class("FixedSizeListType",
 #' @export
 fixed_size_list_of <- function(type, list_size) fixed_size_list__(type, list_size)
 
+#' @rdname data-type
+#' @export
+MapType <- R6Class("MapType",
+  inherit = ListType,
+  active = list(
+    key_field = function() MapType__key_field(self),
+    item_field = function() MapType__item_field(self),
+    key_type = function() MapType__key_type(self),
+    item_type = function() MapType__item_type(self),
+    keys_sorted = function() MapType__keys_sorted(self)
+  )
+)
+
+#' @rdname data-type
+#' @export
+map_of <- function(key_type, item_type, .keys_sorted = FALSE) map__(key_type, item_type, .keys_sorted)
+
 as_type <- function(type, name = "type") {
   # magic so we don't have to mask base::double()
   if (identical(type, double())) {
@@ -469,7 +674,8 @@ canonical_type_str <- function(type_str) {
     time64 = "time64",
     null = "null",
     timestamp = "timestamp",
-    decimal = "decimal128",
+    decimal128 = "decimal128",
+    decimal256 = "decimal256",
     struct = "struct",
     list_of = "list",
     list = "list",
@@ -477,17 +683,19 @@ canonical_type_str <- function(type_str) {
     large_list = "large_list",
     fixed_size_list_of = "fixed_size_list",
     fixed_size_list = "fixed_size_list",
+    map_of = "map",
+    duration = "duration",
     stop("Unrecognized string representation of data type", call. = FALSE)
   )
 }
 
 # vctrs support -----------------------------------------------------------
-str_dup <- function(x, times) {
+duplicate_string <- function(x, times) {
   paste0(rep(x, times = times), collapse = "")
 }
 
 indent <- function(x, n) {
-  pad <- str_dup(" ", n)
+  pad <- duplicate_string(" ", n)
   sapply(x, gsub, pattern = "(\n+)", replacement = paste0("\\1", pad))
 }
 
