@@ -24,7 +24,10 @@ build_features <- c(
 skip_if_not_available <- function(feature) {
   if (feature == "re2") {
     # RE2 does not support valgrind (on purpose): https://github.com/google/re2/issues/177
-    skip_on_valgrind()
+    skip_on_linux_devel()
+  } else if (feature == "snappy") {
+    # Snappy has a UBSan issue: https://github.com/google/snappy/pull/148
+    skip_on_linux_devel()
   }
 
   yes <- feature %in% names(build_features) && build_features[feature]
@@ -34,7 +37,8 @@ skip_if_not_available <- function(feature) {
 }
 
 skip_if_no_pyarrow <- function() {
-  skip_on_valgrind()
+  skip_on_linux_devel()
+  skip_on_os("windows")
 
   skip_if_not_installed("reticulate")
   if (!reticulate::py_module_available("pyarrow")) {
@@ -56,18 +60,20 @@ skip_if_not_running_large_memory_tests <- function() {
   )
 }
 
-skip_on_valgrind <- function() {
-  # This does not actually skip on valgrind because we can't exactly detect it.
-  # Instead, it skips on CRAN when the OS is linux + and the R version is development 
-  # (which is where valgrind is run as of this code)
-  linux_dev <- identical(tolower(Sys.info()[["sysname"]]), "linux") &&
-    grepl("devel", R.version.string)
-
-  if (linux_dev) {
+skip_on_linux_devel <- function() {
+  # Skip when the OS is linux + and the R version is development
+  # helpful for skipping on Valgrind, and the sanitizer checks (clang + gcc) on cran
+  if (on_linux_dev()) {
     skip_on_cran()
   }
 }
 
+skip_if_r_version <- function(r_version) {
+  if (getRversion() <= r_version) {
+    skip(paste("R version:", getRversion()))
+  }
+}
+
 process_is_running <- function(x) {
   cmd <- sprintf("ps aux | grep '%s' | grep -v grep", x)
   tryCatch(system(cmd, ignore.stdout = TRUE) == 0, error = function(e) FALSE)
