@@ -15,39 +15,37 @@
 # specific language governing permissions and limitations
 # under the License.
 
-context("DataType")
-
-test_that("null type works as expected",{
+test_that("null type works as expected", {
   x <- null()
   expect_equal(x$id, 0L)
   expect_equal(x$name, "null")
   expect_equal(x$ToString(), "null")
   expect_true(x == x)
   expect_false(x == int8())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
 })
 
-test_that("boolean type work as expected",{
+test_that("boolean type work as expected", {
   x <- boolean()
   expect_equal(x$id, Type$BOOL)
   expect_equal(x$name, "bool")
   expect_equal(x$ToString(), "bool")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 1L)
 })
 
-test_that("int types works as expected",{
+test_that("int types works as expected", {
   x <- uint8()
   expect_equal(x$id, Type$UINT8)
   expect_equal(x$name, "uint8")
   expect_equal(x$ToString(), "uint8")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 8L)
 
@@ -57,7 +55,7 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "int8")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 8L)
 
@@ -67,7 +65,7 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "uint16")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 16L)
 
@@ -77,7 +75,7 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "int16")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 16L)
 
@@ -87,7 +85,7 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "uint32")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
 
@@ -97,7 +95,7 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "int32")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
 
@@ -107,7 +105,7 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "uint64")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
 
@@ -117,19 +115,19 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "int64")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
 })
 
-test_that("float types work as expected",{
+test_that("float types work as expected", {
   x <- float16()
   expect_equal(x$id, Type$HALF_FLOAT)
   expect_equal(x$name, "halffloat")
   expect_equal(x$ToString(), "halffloat")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 16L)
 
@@ -139,7 +137,7 @@ test_that("float types work as expected",{
   expect_equal(x$ToString(), "float")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
 
@@ -149,19 +147,19 @@ test_that("float types work as expected",{
   expect_equal(x$ToString(), "double")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
 })
 
-test_that("utf8 type works as expected",{
+test_that("utf8 type works as expected", {
   x <- utf8()
   expect_equal(x$id, Type$STRING)
   expect_equal(x$name, "utf8")
   expect_equal(x$ToString(), "string")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
 })
 
@@ -172,7 +170,7 @@ test_that("date types work as expected", {
   expect_equal(x$ToString(), "date32[day]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$unit(), unclass(DateUnit$DAY))
 
@@ -182,7 +180,7 @@ test_that("date types work as expected", {
   expect_equal(x$ToString(), "date64[ms]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$unit(), unclass(DateUnit$MILLI))
 })
@@ -194,7 +192,7 @@ test_that("timestamp type works as expected", {
   expect_equal(x$ToString(), "timestamp[s]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$timezone(), "")
@@ -206,7 +204,7 @@ test_that("timestamp type works as expected", {
   expect_equal(x$ToString(), "timestamp[ms]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$timezone(), "")
@@ -218,7 +216,7 @@ test_that("timestamp type works as expected", {
   expect_equal(x$ToString(), "timestamp[us]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$timezone(), "")
@@ -230,7 +228,7 @@ test_that("timestamp type works as expected", {
   expect_equal(x$ToString(), "timestamp[ns]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$timezone(), "")
@@ -248,7 +246,7 @@ test_that("time32 types work as expected", {
   expect_equal(x$ToString(), "time32[s]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
   expect_equal(x$unit(), unclass(TimeUnit$SECOND))
@@ -259,7 +257,7 @@ test_that("time32 types work as expected", {
   expect_equal(x$ToString(), "time32[ms]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
   expect_equal(x$unit(), unclass(TimeUnit$MILLI))
@@ -272,7 +270,7 @@ test_that("time64 types work as expected", {
   expect_equal(x$ToString(), "time64[us]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$unit(), unclass(TimeUnit$MICRO))
@@ -283,12 +281,36 @@ test_that("time64 types work as expected", {
   expect_equal(x$ToString(), "time64[ns]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$num_fields, 0L)
   expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$unit(), unclass(TimeUnit$NANO))
 })
 
+test_that("duration types work as expected", {
+  x <- duration(TimeUnit$MICRO)
+  expect_equal(x$id, Type$DURATION)
+  expect_equal(x$name, "duration")
+  expect_equal(x$ToString(), "duration[us]")
+  expect_true(x == x)
+  expect_false(x == null())
+  expect_equal(x$num_fields, 0L)
+  expect_equal(x$fields(), list())
+  expect_equal(x$bit_width, 64L)
+  expect_equal(x$unit(), unclass(TimeUnit$MICRO))
+
+  x <- duration(TimeUnit$SECOND)
+  expect_equal(x$id, Type$DURATION)
+  expect_equal(x$name, "duration")
+  expect_equal(x$ToString(), "duration[s]")
+  expect_true(x == x)
+  expect_false(x == null())
+  expect_equal(x$num_fields, 0L)
+  expect_equal(x$fields(), list())
+  expect_equal(x$bit_width, 64L)
+  expect_equal(x$unit(), unclass(TimeUnit$SECOND))
+})
+
 test_that("time type unit validation", {
   expect_equal(time32(TimeUnit$SECOND), time32("s"))
   expect_equal(time32(TimeUnit$MILLI), time32("ms"))
@@ -303,6 +325,13 @@ test_that("time type unit validation", {
   expect_error(time64(4), '"unit" should be one of 3 or 2')
   expect_error(time64(NULL), '"unit" should be one of "ns" or "us"')
   expect_match_arg_error(time64("years"))
+
+  expect_equal(duration(TimeUnit$NANO), duration("n"))
+  expect_equal(duration(TimeUnit$MICRO), duration("us"))
+  expect_equal(duration(), duration(TimeUnit$SECOND))
+  expect_error(duration(4), '"unit" should be one of 0, 1, 2, or 3')
+  expect_error(duration(NULL), '"unit" should be one of "s", "ms", "us", or "ns"')
+  expect_match_arg_error(duration("years"))
 })
 
 test_that("timestamp type input validation", {
@@ -329,15 +358,53 @@ test_that("list type works as expected", {
   expect_equal(x$ToString(), "list<item: int32>")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 1L)
+  expect_equal(x$num_fields, 1L)
   expect_equal(
-    x$fields(),
-    list(field("item", int32()))
+    x$fields()[[1]],
+    field("item", int32())
   )
   expect_equal(x$value_type, int32())
   expect_equal(x$value_field, field("item", int32()))
 })
 
+test_that("map type works as expected", {
+  x <- map_of(int32(), utf8())
+  expect_equal(x$id, Type$MAP)
+  expect_equal(x$name, "map")
+  expect_equal(x$ToString(), "map<int32, string>")
+  expect_true(x == x)
+  expect_false(x == null())
+  expect_equal(
+    x$key_field,
+    field("key", int32(), nullable = FALSE)
+  )
+  expect_equal(
+    x$item_field,
+    field("value", utf8())
+  )
+  expect_equal(x$key_type, int32())
+  expect_equal(x$item_type, string())
+  # TODO: (ARROW-15102): Enable constructing StructTypes with non-nullable fields, so
+  # we can make this comparison:
+  # expect_equal(x$value_type, struct(key = x$key_field, value = x$item_field)) # nolint
+  expect_false(x$keys_sorted)
+})
+
+test_that("map type validates arguments", {
+  expect_error(map_of(field("key", int32(), nullable = TRUE), utf8()),
+               "cannot be nullable")
+  expect_error(map_of(1L, utf8()), "must be a DataType or Field")
+  expect_error(map_of(int32(), 1L), "must be a DataType or Field")
+
+  # field construction
+  ty <- map_of(field("the_keys", int32(), nullable = FALSE),
+               field("my_values", utf8(), nullable = FALSE))
+  expect_equal(ty$key_field$name, "the_keys")
+  expect_equal(ty$item_field$name, "my_values")
+  expect_equal(ty$key_field$nullable, FALSE)
+  expect_equal(ty$item_field$nullable, FALSE)
+})
+
 test_that("struct type works as expected", {
   x <- struct(x = int32(), y = boolean())
   expect_equal(x$id, Type$STRUCT)
@@ -345,10 +412,14 @@ test_that("struct type works as expected", {
   expect_equal(x$ToString(), "struct<x: int32, y: bool>")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_fields(), 2L)
+  expect_equal(x$num_fields, 2L)
   expect_equal(
-    x$fields(),
-    list(field("x", int32()), field("y", boolean()))
+    x$fields()[[1]],
+    field("x", int32())
+  )
+  expect_equal(
+    x$fields()[[2]],
+    field("y", boolean())
   )
   expect_equal(x$GetFieldIndex("x"), 0L)
   expect_equal(x$GetFieldIndex("y"), 1L)
@@ -383,26 +454,54 @@ test_that("DictionaryType validation", {
 })
 
 test_that("decimal type and validation", {
-  expect_error(decimal())
-  expect_error(decimal("four"), '"precision" must be an integer')
-  expect_error(decimal(4))
-  expect_error(decimal(4, "two"), '"scale" must be an integer')
-  expect_error(decimal(NA, 2), '"precision" must be an integer')
-  expect_error(decimal(0, 2), "Invalid: Decimal precision out of range: 0")
-  expect_error(decimal(100, 2), "Invalid: Decimal precision out of range: 100")
-  expect_error(decimal(4, NA), '"scale" must be an integer')
+  expect_r6_class(decimal(4, 2), "Decimal128Type")
+  expect_r6_class(decimal(39, 2), "Decimal256Type")
 
-  expect_is(decimal(4, 2), "Decimal128Type")
+  expect_error(decimal("four"), "`precision` must be an integer")
+  expect_error(decimal(4, "two"), "`scale` must be an integer")
+  expect_error(decimal(NA, 2), "`precision` must be an integer")
+  expect_error(decimal(4, NA), "`scale` must be an integer")
+  # TODO remove precision range tests below once functionality is tested in C++ (ARROW-15162)
+  expect_error(decimal(0, 2), "Invalid: Decimal precision out of range [1, 38]: 0", fixed = TRUE)
+  expect_error(decimal(100, 2), "Invalid: Decimal precision out of range [1, 76]: 100", fixed = TRUE)
 
+  # decimal() creates either decimal128 or decimal256 based on precision
+  expect_identical(class(decimal(38, 2)), class(decimal128(38, 2)))
+  expect_identical(class(decimal(39, 2)), class(decimal256(38, 2)))
+
+  expect_r6_class(decimal128(4, 2), "Decimal128Type")
+
+  expect_error(decimal128("four"), "`precision` must be an integer")
+  expect_error(decimal128(4, "two"), "`scale` must be an integer")
+  expect_error(decimal128(NA, 2), "`precision` must be an integer")
+  expect_error(decimal128(4, NA), "`scale` must be an integer")
+  expect_error(decimal128(3:4, NA), "`precision` must have size 1. not size 2")
+  expect_error(decimal128(4, 2:3), "`scale` must have size 1. not size 2")
+  # TODO remove precision range tests below once functionality is tested in C++ (ARROW-15162)
+  expect_error(decimal128(0, 2), "Invalid: Decimal precision out of range [1, 38]: 0", fixed = TRUE)
+  expect_error(decimal128(100, 2), "Invalid: Decimal precision out of range [1, 38]: 100", fixed = TRUE)
+
+
+  expect_r6_class(decimal256(4, 2), "Decimal256Type")
+
+  expect_error(decimal256("four"), "`precision` must be an integer")
+  expect_error(decimal256(4, "two"), "`scale` must be an integer")
+  expect_error(decimal256(NA, 2), "`precision` must be an integer")
+  expect_error(decimal256(4, NA), "`scale` must be an integer")
+  expect_error(decimal256(3:4, NA), "`precision` must have size 1. not size 2")
+  expect_error(decimal256(4, 2:3), "`scale` must have size 1. not size 2")
+  # TODO remove precision range tests below once functionality is tested in C++ (ARROW-15162)
+  expect_error(decimal256(0, 2), "Invalid: Decimal precision out of range [1, 76]: 0", fixed = TRUE)
+  expect_error(decimal256(100, 2), "Invalid: Decimal precision out of range [1, 76]: 100", fixed = TRUE)
 })
 
 test_that("Binary", {
-  expect_is(binary(), "Binary")
+  expect_r6_class(binary(), "Binary")
   expect_equal(binary()$ToString(), "binary")
 })
 
 test_that("FixedSizeBinary", {
-  expect_is(fixed_size_binary(4), "FixedSizeBinary")
+  expect_r6_class(fixed_size_binary(4), "FixedSizeBinary")
   expect_equal(fixed_size_binary(4)$ToString(), "fixed_size_binary[4]")
 
   # input validation
@@ -411,3 +510,93 @@ test_that("FixedSizeBinary", {
   expect_error(fixed_size_binary("four"))
   expect_error(fixed_size_binary(c(2, 4)))
 })
+
+test_that("DataType to C-interface", {
+  datatype <- timestamp("ms", timezone = "Pacific/Marquesas")
+
+  # export the datatype via the C-interface
+  ptr <- allocate_arrow_schema()
+  datatype$export_to_c(ptr)
+
+  # then import it and check that the roundtripped value is the same
+  circle <- DataType$import_from_c(ptr)
+  expect_equal(circle, datatype)
+
+  # must clean up the pointer or we leak
+  delete_arrow_schema(ptr)
+})
+
+test_that("DataType$code()", {
+  expect_code_roundtrip(int8())
+  expect_code_roundtrip(uint8())
+  expect_code_roundtrip(int16())
+  expect_code_roundtrip(uint16())
+  expect_code_roundtrip(int32())
+  expect_code_roundtrip(uint32())
+  expect_code_roundtrip(int64())
+  expect_code_roundtrip(uint64())
+
+  expect_code_roundtrip(float16())
+  expect_code_roundtrip(float32())
+  expect_code_roundtrip(float64())
+
+  expect_code_roundtrip(null())
+
+  expect_code_roundtrip(boolean())
+  expect_code_roundtrip(utf8())
+  expect_code_roundtrip(large_utf8())
+
+  expect_code_roundtrip(binary())
+  expect_code_roundtrip(large_binary())
+  expect_code_roundtrip(fixed_size_binary(byte_width = 91))
+
+  expect_code_roundtrip(date32())
+  expect_code_roundtrip(date64())
+
+  expect_code_roundtrip(time32())
+  expect_code_roundtrip(time32(unit = "ms"))
+  expect_code_roundtrip(time32(unit = "s"))
+
+  expect_code_roundtrip(time64())
+  expect_code_roundtrip(time64(unit = "ns"))
+  expect_code_roundtrip(time64(unit = "us"))
+
+  expect_code_roundtrip(timestamp())
+  expect_code_roundtrip(timestamp(unit = "s"))
+  expect_code_roundtrip(timestamp(unit = "ms"))
+  expect_code_roundtrip(timestamp(unit = "ns"))
+  expect_code_roundtrip(timestamp(unit = "us"))
+
+  expect_code_roundtrip(timestamp(unit = "s", timezone = "CET"))
+  expect_code_roundtrip(timestamp(unit = "ms", timezone = "CET"))
+  expect_code_roundtrip(timestamp(unit = "ns", timezone = "CET"))
+  expect_code_roundtrip(timestamp(unit = "us", timezone = "CET"))
+
+  expect_code_roundtrip(decimal(precision = 3, scale = 5))
+
+  expect_code_roundtrip(
+    struct(a = int32(), b = struct(c = list_of(utf8())), d = float64())
+  )
+
+  expect_code_roundtrip(list_of(int32()))
+  expect_code_roundtrip(large_list_of(int32()))
+  expect_code_roundtrip(
+    fixed_size_list_of(int32(), list_size = 7L)
+  )
+
+  expect_code_roundtrip(dictionary())
+  expect_code_roundtrip(dictionary(index_type = int8()))
+  expect_code_roundtrip(dictionary(index_type = int8(), value_type = large_utf8()))
+  expect_code_roundtrip(dictionary(index_type = int8(), ordered = TRUE))
+
+  skip("until rlang 1.0")
+  expect_snapshot({
+    (expect_error(
+      DayTimeInterval__initialize()$code()
+    ))
+    (expect_error(
+      struct(a = DayTimeInterval__initialize())$code()
+    ))
+  })
+
+})
