@@ -19,10 +19,13 @@
 
 #include <algorithm>
 #include <atomic>
+#include <chrono>
 #include <condition_variable>
 #include <functional>
+#include <memory>
 #include <mutex>
 #include <sstream>
+#include <thread>
 #include <unordered_map>
 #include <utility>
 
@@ -41,10 +44,12 @@
 #include <aws/core/auth/AWSCredentials.h>
 #include <aws/core/auth/AWSCredentialsProviderChain.h>
 #include <aws/core/auth/STSCredentialsProvider.h>
+#include <aws/core/client/DefaultRetryStrategy.h>
 #include <aws/core/client/RetryStrategy.h>
 #include <aws/core/http/HttpResponse.h>
 #include <aws/core/utils/logging/ConsoleLogSystem.h>
 #include <aws/core/utils/stream/PreallocatedStreamBuf.h>
+#include <aws/core/utils/xml/XmlSerializer.h>
 #include <aws/identity-management/auth/STSAssumeRoleCredentialsProvider.h>
 #include <aws/s3/S3Client.h>
 #include <aws/s3/model/AbortMultipartUploadRequest.h>
@@ -205,6 +210,10 @@ bool S3ProxyOptions::Equals(const S3ProxyOptions& other) const {
 // -----------------------------------------------------------------------
 // S3Options implementation
 
+S3Options::S3Options() {
+  DCHECK(aws_initialized.load()) << "Must initialize S3 before using S3Options";
+}
+
 void S3Options::ConfigureDefaultCredentials() {
   credentials_provider =
       std::make_shared<Aws::Auth::DefaultAWSCredentialsProviderChain>();
@@ -350,7 +359,7 @@ Result<S3Options> S3Options::FromUri(const Uri& uri, std::string* out_path) {
 
   if (!region_set && !bucket.empty() && options.endpoint_override.empty()) {
     // XXX Should we use a dedicated resolver with the given credentials?
-    ARROW_ASSIGN_OR_RAISE(options.region, ResolveBucketRegion(bucket));
+    ARROW_ASSIGN_OR_RAISE(options.region, ResolveS3BucketRegion(bucket));
   }
 
   return options;
@@ -393,6 +402,10 @@ struct S3Path {
   std::vector<std::string> key_parts;
 
   static Result<S3Path> FromString(const std::string& s) {
+    if (internal::IsLikelyUri(s)) {
+      return Status::Invalid(
+          "Expected an S3 object path of the form 'bucket/key...', got a URI: '", s, "'");
+    }
     const auto src = internal::RemoveTrailingSlash(s);
     auto first_sep = src.find_first_of(kSep);
     if (first_sep == 0) {
@@ -406,14 +419,14 @@ struct S3Path {
     path.bucket = std::string(src.substr(0, first_sep));
     path.key = std::string(src.substr(first_sep + 1));
     path.key_parts = internal::SplitAbstractPath(path.key);
-    RETURN_NOT_OK(Validate(&path));
+    RETURN_NOT_OK(Validate(path));
     return path;
   }
 
-  static Status Validate(const S3Path* path) {
-    auto result = internal::ValidateAbstractPathParts(path->key_parts);
+  static Status Validate(const S3Path& path) {
+    auto result = internal::ValidateAbstractPathParts(path.key_parts);
     if (!result.ok()) {
-      return Status::Invalid(result.message(), " in path ", path->full_path);
+      return Status::Invalid(result.message(), " in path ", path.full_path);
     } else {
       return result;
     }
@@ -563,6 +576,98 @@ class S3Client : public Aws::S3::S3Client {
     req.SetBucket(ToAwsString(bucket));
     return GetBucketRegion(req);
   }
+
+  S3Model::CompleteMultipartUploadOutcome CompleteMultipartUploadWithErrorFixup(
+      S3Model::CompleteMultipartUploadRequest&& request) const {
+    // CompletedMultipartUpload can return a 200 OK response with an error
+    // encoded in the response body, in which case we should either retry
+    // or propagate the error to the user (see
+    // https://docs.aws.amazon.com/AmazonS3/latest/API/API_CompleteMultipartUpload.html).
+    //
+    // Unfortunately the AWS SDK doesn't detect such situations but lets them
+    // return successfully (see https://github.com/aws/aws-sdk-cpp/issues/658).
+    //
+    // We work around the issue by registering a DataReceivedEventHandler
+    // which parses the XML response for embedded errors.
+
+    util::optional<AWSError<Aws::Client::CoreErrors>> aws_error;
+
+    auto handler = [&](const Aws::Http::HttpRequest* http_req,
+                       Aws::Http::HttpResponse* http_resp,
+                       long long) {  // NOLINT runtime/int
+      auto& stream = http_resp->GetResponseBody();
+      const auto pos = stream.tellg();
+      const auto doc = Aws::Utils::Xml::XmlDocument::CreateFromXmlStream(stream);
+      // Rewind stream for later
+      stream.clear();
+      stream.seekg(pos);
+
+      if (doc.WasParseSuccessful()) {
+        auto root = doc.GetRootElement();
+        if (!root.IsNull()) {
+          // Detect something that looks like an abnormal CompletedMultipartUpload
+          // response.
+          if (root.GetName() != "CompleteMultipartUploadResult" ||
+              !root.FirstChild("Error").IsNull() || !root.FirstChild("Errors").IsNull()) {
+            // Make sure the error marshaller doesn't see a 200 OK
+            http_resp->SetResponseCode(
+                Aws::Http::HttpResponseCode::INTERNAL_SERVER_ERROR);
+            aws_error = GetErrorMarshaller()->Marshall(*http_resp);
+            // Rewind stream for later
+            stream.clear();
+            stream.seekg(pos);
+          }
+        }
+      }
+    };
+
+    request.SetDataReceivedEventHandler(std::move(handler));
+
+    // We don't have access to the configured AWS retry strategy
+    // (m_retryStrategy is a private member of AwsClient), so don't use that.
+    std::unique_ptr<Aws::Client::RetryStrategy> retry_strategy;
+    if (s3_retry_strategy_) {
+      retry_strategy.reset(new WrappedRetryStrategy(s3_retry_strategy_));
+    } else {
+      // Note that DefaultRetryStrategy, unlike StandardRetryStrategy,
+      // has empty definitions for RequestBookkeeping() and GetSendToken(),
+      // which simplifies the code below.
+      retry_strategy.reset(new Aws::Client::DefaultRetryStrategy());
+    }
+
+    for (int32_t retries = 0;; retries++) {
+      aws_error.reset();
+      auto outcome = Aws::S3::S3Client::S3Client::CompleteMultipartUpload(request);
+      if (!outcome.IsSuccess()) {
+        // Error returned in HTTP headers (or client failure)
+        return outcome;
+      }
+      if (!aws_error.has_value()) {
+        // Genuinely successful outcome
+        return outcome;
+      }
+
+      const bool should_retry = retry_strategy->ShouldRetry(*aws_error, retries);
+
+      ARROW_LOG(WARNING)
+          << "CompletedMultipartUpload got error embedded in a 200 OK response: "
+          << aws_error->GetExceptionName() << " (\"" << aws_error->GetMessage()
+          << "\"), retry = " << should_retry;
+
+      if (!should_retry) {
+        break;
+      }
+      const auto delay = std::chrono::milliseconds(
+          retry_strategy->CalculateDelayBeforeNextRetry(*aws_error, retries));
+      std::this_thread::sleep_for(delay);
+    }
+
+    DCHECK(aws_error.has_value());
+    auto s3_error = AWSError<S3Errors>(std::move(aws_error).value());
+    return S3Model::CompleteMultipartUploadOutcome(std::move(s3_error));
+  }
+
+  std::shared_ptr<S3RetryStrategy> s3_retry_strategy_;
 };
 
 // In AWS SDK < 1.8, Aws::Client::ClientConfiguration::followRedirects is a bool.
@@ -589,7 +694,8 @@ class ClientBuilder {
 
   Aws::Client::ClientConfiguration* mutable_config() { return &client_config_; }
 
-  Result<std::shared_ptr<S3Client>> BuildClient() {
+  Result<std::shared_ptr<S3Client>> BuildClient(
+      util::optional<io::IOContext> io_context = util::nullopt) {
     credentials_provider_ = options_.credentials_provider;
     if (!options_.region.empty()) {
       client_config_.region = ToAwsString(options_.region);
@@ -617,7 +723,7 @@ class ClientBuilder {
 
     const bool use_virtual_addressing = options_.endpoint_override.empty();
 
-    /// Set proxy options if provided
+    // Set proxy options if provided
     if (!options_.proxy_options.scheme.empty()) {
       if (options_.proxy_options.scheme == "http") {
         client_config_.proxyScheme = Aws::Http::Scheme::HTTP;
@@ -641,10 +747,17 @@ class ClientBuilder {
       client_config_.proxyPassword = ToAwsString(options_.proxy_options.password);
     }
 
-    return std::make_shared<S3Client>(
+    if (io_context) {
+      // TODO: Once ARROW-15035 is done we can get rid of the "at least 25" fallback
+      client_config_.maxConnections = std::max(io_context->executor()->GetCapacity(), 25);
+    }
+
+    auto client = std::make_shared<S3Client>(
         credentials_provider_, client_config_,
         Aws::Client::AWSAuthV4Signer::PayloadSigningPolicy::Never,
         use_virtual_addressing);
+    client->s3_retry_strategy_ = options_.retry_strategy;
+    return client;
   }
 
   const S3Options& options() const { return options_; }
@@ -1021,9 +1134,8 @@ class ObjectOutputStream final : public io::OutputStream {
   struct UploadState;
 
  public:
-  ObjectOutputStream(std::shared_ptr<Aws::S3::S3Client> client,
-                     const io::IOContext& io_context, const S3Path& path,
-                     const S3Options& options,
+  ObjectOutputStream(std::shared_ptr<S3Client> client, const io::IOContext& io_context,
+                     const S3Path& path, const S3Options& options,
                      const std::shared_ptr<const KeyValueMetadata>& metadata)
       : client_(std::move(client)),
         io_context_(io_context),
@@ -1049,6 +1161,13 @@ class ObjectOutputStream final : public io::OutputStream {
       RETURN_NOT_OK(SetObjectMetadata(default_metadata_, &req));
     }
 
+    // If we do not set anything then the SDK will default to application/xml
+    // which confuses some tools (https://github.com/apache/arrow/issues/11934)
+    // So we instead default to application/octet-stream which is less misleading
+    if (!req.ContentTypeHasBeenSet()) {
+      req.SetContentType("application/octet-stream");
+    }
+
     auto outcome = client_->CreateMultipartUpload(req);
     if (!outcome.IsSuccess()) {
       return ErrorToStatus(
@@ -1088,9 +1207,12 @@ class ObjectOutputStream final : public io::OutputStream {
   // OutputStream interface
 
   Status Close() override {
-    if (closed_) {
-      return Status::OK();
-    }
+    auto fut = CloseAsync();
+    return fut.status();
+  }
+
+  Future<> CloseAsync() override {
+    if (closed_) return Status::OK();
 
     if (current_part_) {
       // Upload last part
@@ -1103,32 +1225,32 @@ class ObjectOutputStream final : public io::OutputStream {
     }
 
     // Wait for in-progress uploads to finish (if async writes are enabled)
-    RETURN_NOT_OK(Flush());
+    return FlushAsync().Then([this]() {
+      // At this point, all part uploads have finished successfully
+      DCHECK_GT(part_number_, 1);
+      DCHECK_EQ(upload_state_->completed_parts.size(),
+                static_cast<size_t>(part_number_ - 1));
 
-    // At this point, all part uploads have finished successfully
-    DCHECK_GT(part_number_, 1);
-    DCHECK_EQ(upload_state_->completed_parts.size(),
-              static_cast<size_t>(part_number_ - 1));
+      S3Model::CompletedMultipartUpload completed_upload;
+      completed_upload.SetParts(upload_state_->completed_parts);
+      S3Model::CompleteMultipartUploadRequest req;
+      req.SetBucket(ToAwsString(path_.bucket));
+      req.SetKey(ToAwsString(path_.key));
+      req.SetUploadId(upload_id_);
+      req.SetMultipartUpload(std::move(completed_upload));
 
-    S3Model::CompletedMultipartUpload completed_upload;
-    completed_upload.SetParts(upload_state_->completed_parts);
-    S3Model::CompleteMultipartUploadRequest req;
-    req.SetBucket(ToAwsString(path_.bucket));
-    req.SetKey(ToAwsString(path_.key));
-    req.SetUploadId(upload_id_);
-    req.SetMultipartUpload(std::move(completed_upload));
+      auto outcome = client_->CompleteMultipartUploadWithErrorFixup(std::move(req));
+      if (!outcome.IsSuccess()) {
+        return ErrorToStatus(
+            std::forward_as_tuple("When completing multiple part upload for key '",
+                                  path_.key, "' in bucket '", path_.bucket, "': "),
+            outcome.GetError());
+      }
 
-    auto outcome = client_->CompleteMultipartUpload(req);
-    if (!outcome.IsSuccess()) {
-      return ErrorToStatus(
-          std::forward_as_tuple("When completing multiple part upload for key '",
-                                path_.key, "' in bucket '", path_.bucket, "': "),
-          outcome.GetError());
-    }
-
-    client_ = nullptr;
-    closed_ = true;
-    return Status::OK();
+      client_ = nullptr;
+      closed_ = true;
+      return Status::OK();
+    });
   }
 
   bool closed() const override { return closed_; }
@@ -1181,14 +1303,17 @@ class ObjectOutputStream final : public io::OutputStream {
   }
 
   Status Flush() override {
+    auto fut = FlushAsync();
+    return fut.status();
+  }
+
+  Future<> FlushAsync() {
     if (closed_) {
       return Status::Invalid("Operation on closed stream");
     }
     // Wait for background writes to finish
     std::unique_lock<std::mutex> lock(upload_state_->mutex);
-    upload_state_->cv.wait(lock,
-                           [this]() { return upload_state_->parts_in_progress == 0; });
-    return upload_state_->status;
+    return upload_state_->pending_parts_completed;
   }
 
   // Upload-related helpers
@@ -1235,7 +1360,9 @@ class ObjectOutputStream final : public io::OutputStream {
 
       {
         std::unique_lock<std::mutex> lock(upload_state_->mutex);
-        ++upload_state_->parts_in_progress;
+        if (upload_state_->parts_in_progress++ == 0) {
+          upload_state_->pending_parts_completed = Future<>::Make();
+        }
       }
       auto client = client_;
       ARROW_ASSIGN_OR_RAISE(auto fut, SubmitIO(io_context_, [client, req]() {
@@ -1286,7 +1413,7 @@ class ObjectOutputStream final : public io::OutputStream {
     }
     // Notify completion
     if (--state->parts_in_progress == 0) {
-      state->cv.notify_all();
+      state->pending_parts_completed.MarkFinished(state->status);
     }
   }
 
@@ -1314,7 +1441,7 @@ class ObjectOutputStream final : public io::OutputStream {
   }
 
  protected:
-  std::shared_ptr<Aws::S3::S3Client> client_;
+  std::shared_ptr<S3Client> client_;
   const io::IOContext io_context_;
   const S3Path path_;
   const std::shared_ptr<const KeyValueMetadata> metadata_;
@@ -1333,10 +1460,10 @@ class ObjectOutputStream final : public io::OutputStream {
   // in the completion handler.
   struct UploadState {
     std::mutex mutex;
-    std::condition_variable cv;
     Aws::Vector<S3Model::CompletedPart> completed_parts;
     int64_t parts_in_progress = 0;
     Status status;
+    Future<> pending_parts_completed = Future<>::MakeFinished(Status::OK());
   };
   std::shared_ptr<UploadState> upload_state_;
 };
@@ -1503,7 +1630,7 @@ class S3FileSystem::Impl : public std::enable_shared_from_this<S3FileSystem::Imp
  public:
   ClientBuilder builder_;
   io::IOContext io_context_;
-  std::shared_ptr<Aws::S3::S3Client> client_;
+  std::shared_ptr<S3Client> client_;
   util::optional<S3Backend> backend_;
 
   const int32_t kListObjectsMaxKeys = 1000;
@@ -1515,7 +1642,7 @@ class S3FileSystem::Impl : public std::enable_shared_from_this<S3FileSystem::Imp
   explicit Impl(S3Options options, io::IOContext io_context)
       : builder_(std::move(options)), io_context_(io_context) {}
 
-  Status Init() { return builder_.BuildClient().Value(&client_); }
+  Status Init() { return builder_.BuildClient(io_context_).Value(&client_); }
 
   const S3Options& options() const { return builder_.options(); }
 
@@ -1612,7 +1739,27 @@ class S3FileSystem::Impl : public std::enable_shared_from_this<S3FileSystem::Imp
   // a non-empty "directory".  This is a Minio-specific quirk, but we need
   // to handle it for unit testing.
 
-  Status IsEmptyDirectory(const std::string& bucket, const std::string& key, bool* out) {
+  // If this method is called after HEAD on "bucket/key" already returned a 404,
+  // can pass the given outcome to spare a spurious HEAD call.
+  Result<bool> IsEmptyDirectory(
+      const std::string& bucket, const std::string& key,
+      const S3Model::HeadObjectOutcome* previous_outcome = nullptr) {
+    if (previous_outcome) {
+      // Fetch the backend from the previous error
+      DCHECK(!previous_outcome->IsSuccess());
+      if (!backend_) {
+        SaveBackend(previous_outcome->GetError());
+        DCHECK(backend_);
+      }
+      if (backend_ != S3Backend::Minio) {
+        // HEAD already returned a 404, nothing more to do
+        return false;
+      }
+    }
+
+    // We come here in one of two situations:
+    // - we don't know the backend and there is no previous outcome
+    // - the backend is Minio
     S3Model::HeadObjectRequest req;
     req.SetBucket(ToAwsString(bucket));
     if (backend_ && *backend_ == S3Backend::Minio) {
@@ -1624,31 +1771,30 @@ class S3FileSystem::Impl : public std::enable_shared_from_this<S3FileSystem::Imp
 
     auto outcome = client_->HeadObject(req);
     if (outcome.IsSuccess()) {
-      *out = true;
-      return Status::OK();
+      return true;
     }
     if (!backend_) {
       SaveBackend(outcome.GetError());
       DCHECK(backend_);
       if (*backend_ == S3Backend::Minio) {
         // Try again with separator-terminated key (see above)
-        return IsEmptyDirectory(bucket, key, out);
+        return IsEmptyDirectory(bucket, key);
       }
     }
     if (IsNotFound(outcome.GetError())) {
-      *out = false;
-      return Status::OK();
+      return false;
     }
     return ErrorToStatus(std::forward_as_tuple("When reading information for key '", key,
                                                "' in bucket '", bucket, "': "),
                          outcome.GetError());
   }
 
-  Status IsEmptyDirectory(const S3Path& path, bool* out) {
-    return IsEmptyDirectory(path.bucket, path.key, out);
+  Result<bool> IsEmptyDirectory(
+      const S3Path& path, const S3Model::HeadObjectOutcome* previous_outcome = nullptr) {
+    return IsEmptyDirectory(path.bucket, path.key, previous_outcome);
   }
 
-  Status IsNonEmptyDirectory(const S3Path& path, bool* out) {
+  Result<bool> IsNonEmptyDirectory(const S3Path& path) {
     S3Model::ListObjectsV2Request req;
     req.SetBucket(ToAwsString(path.bucket));
     req.SetPrefix(ToAwsString(path.key) + kSep);
@@ -1656,12 +1802,12 @@ class S3FileSystem::Impl : public std::enable_shared_from_this<S3FileSystem::Imp
     req.SetMaxKeys(1);
     auto outcome = client_->ListObjectsV2(req);
     if (outcome.IsSuccess()) {
-      *out = outcome.GetResult().GetKeyCount() > 0;
-      return Status::OK();
+      const S3Model::ListObjectsV2Result& r = outcome.GetResult();
+      // In some cases, there may be 0 keys but some prefixes
+      return r.GetKeyCount() > 0 || !r.GetCommonPrefixes().empty();
     }
     if (IsNotFound(outcome.GetError())) {
-      *out = false;
-      return Status::OK();
+      return false;
     }
     return ErrorToStatus(
         std::forward_as_tuple("When listing objects under key '", path.key,
@@ -1720,8 +1866,8 @@ class S3FileSystem::Impl : public std::enable_shared_from_this<S3FileSystem::Imp
       // If no contents were found, perhaps it's an empty "directory",
       // or perhaps it's a nonexistent entry.  Check.
       if (is_empty && !allow_not_found) {
-        bool is_actually_empty;
-        RETURN_NOT_OK(impl->IsEmptyDirectory(bucket, key, &is_actually_empty));
+        ARROW_ASSIGN_OR_RAISE(bool is_actually_empty,
+                              impl->IsEmptyDirectory(bucket, key));
         if (!is_actually_empty) {
           return PathNotFound(bucket, key);
         }
@@ -1820,37 +1966,44 @@ class S3FileSystem::Impl : public std::enable_shared_from_this<S3FileSystem::Imp
     return gen;
   }
 
-  Status WalkForDeleteDir(const std::string& bucket, const std::string& key,
-                          std::vector<std::string>* file_keys,
-                          std::vector<std::string>* dir_keys) {
-    auto handle_results = [&](const std::string& prefix,
-                              const S3Model::ListObjectsV2Result& result) -> Status {
+  struct WalkResult {
+    std::vector<std::string> file_keys;
+    std::vector<std::string> dir_keys;
+  };
+  Future<std::shared_ptr<WalkResult>> WalkForDeleteDirAsync(const std::string& bucket,
+                                                            const std::string& key) {
+    auto state = std::make_shared<WalkResult>();
+
+    auto handle_results = [state](const std::string& prefix,
+                                  const S3Model::ListObjectsV2Result& result) -> Status {
       // Walk "files"
-      file_keys->reserve(file_keys->size() + result.GetContents().size());
+      state->file_keys.reserve(state->file_keys.size() + result.GetContents().size());
       for (const auto& obj : result.GetContents()) {
-        file_keys->emplace_back(FromAwsString(obj.GetKey()));
+        state->file_keys.emplace_back(FromAwsString(obj.GetKey()));
       }
       // Walk "directories"
-      dir_keys->reserve(dir_keys->size() + result.GetCommonPrefixes().size());
+      state->dir_keys.reserve(state->dir_keys.size() + result.GetCommonPrefixes().size());
       for (const auto& prefix : result.GetCommonPrefixes()) {
-        dir_keys->emplace_back(FromAwsString(prefix.GetPrefix()));
+        state->dir_keys.emplace_back(FromAwsString(prefix.GetPrefix()));
       }
       return Status::OK();
     };
 
-    auto handle_error = [&](const AWSError<S3Errors>& error) -> Status {
+    auto handle_error = [=](const AWSError<S3Errors>& error) -> Status {
       return ErrorToStatus(std::forward_as_tuple("When listing objects under key '", key,
                                                  "' in bucket '", bucket, "': "),
                            error);
     };
 
-    auto handle_recursion = [&](int32_t nesting_depth) -> Result<bool> {
-      RETURN_NOT_OK(CheckNestingDepth(nesting_depth));
+    auto self = shared_from_this();
+    auto handle_recursion = [self](int32_t nesting_depth) -> Result<bool> {
+      RETURN_NOT_OK(self->CheckNestingDepth(nesting_depth));
       return true;  // Recurse
     };
 
-    return TreeWalker::Walk(client_, io_context_, bucket, key, kListObjectsMaxKeys,
-                            handle_results, handle_error, handle_recursion);
+    return TreeWalker::WalkAsync(client_, io_context_, bucket, key, kListObjectsMaxKeys,
+                                 handle_results, handle_error, handle_recursion)
+        .Then([state]() { return state; });
   }
 
   // Delete multiple objects at once
@@ -1908,24 +2061,28 @@ class S3FileSystem::Impl : public std::enable_shared_from_this<S3FileSystem::Imp
     return DeleteObjectsAsync(bucket, keys).status();
   }
 
-  Status DeleteDirContents(const std::string& bucket, const std::string& key) {
-    std::vector<std::string> file_keys;
-    std::vector<std::string> dir_keys;
-    RETURN_NOT_OK(WalkForDeleteDir(bucket, key, &file_keys, &dir_keys));
-    if (file_keys.empty() && dir_keys.empty() && !key.empty()) {
-      // No contents found, is it an empty directory?
-      bool exists = false;
-      RETURN_NOT_OK(IsEmptyDirectory(bucket, key, &exists));
-      if (!exists) {
-        return PathNotFound(bucket, key);
-      }
-    }
-    // First delete all "files", then delete all child "directories"
-    RETURN_NOT_OK(DeleteObjects(bucket, file_keys));
-    // Delete directories in reverse lexicographic order, to ensure children
-    // are deleted before their parents (Minio).
-    std::sort(dir_keys.rbegin(), dir_keys.rend());
-    return DeleteObjects(bucket, dir_keys);
+  Future<> DeleteDirContentsAsync(const std::string& bucket, const std::string& key) {
+    auto self = shared_from_this();
+    return WalkForDeleteDirAsync(bucket, key)
+        .Then([bucket, key,
+               self](const std::shared_ptr<WalkResult>& discovered) -> Future<> {
+          if (discovered->file_keys.empty() && discovered->dir_keys.empty() &&
+              !key.empty()) {
+            // No contents found, is it an empty directory?
+            ARROW_ASSIGN_OR_RAISE(bool exists, self->IsEmptyDirectory(bucket, key));
+            if (!exists) {
+              return PathNotFound(bucket, key);
+            }
+          }
+          // First delete all "files", then delete all child "directories"
+          return self->DeleteObjectsAsync(bucket, discovered->file_keys)
+              .Then([bucket, discovered, self]() {
+                // Delete directories in reverse lexicographic order, to ensure children
+                // are deleted before their parents (Minio).
+                std::sort(discovered->dir_keys.rbegin(), discovered->dir_keys.rend());
+                return self->DeleteObjectsAsync(bucket, discovered->dir_keys);
+              });
+        });
   }
 
   Status EnsureDirectoryExists(const S3Path& path) {
@@ -2078,14 +2235,13 @@ Result<FileInfo> S3FileSystem::GetFileInfo(const std::string& s) {
           outcome.GetError());
     }
     // Not found => perhaps it's an empty "directory"
-    bool is_dir = false;
-    RETURN_NOT_OK(impl_->IsEmptyDirectory(path, &is_dir));
+    ARROW_ASSIGN_OR_RAISE(bool is_dir, impl_->IsEmptyDirectory(path, &outcome));
     if (is_dir) {
       info.set_type(FileType::Directory);
       return info;
     }
     // Not found => perhaps it's a non-empty "directory"
-    RETURN_NOT_OK(impl_->IsNonEmptyDirectory(path, &is_dir));
+    ARROW_ASSIGN_OR_RAISE(is_dir, impl_->IsNonEmptyDirectory(path));
     if (is_dir) {
       info.set_type(FileType::Directory);
     } else {
@@ -2192,10 +2348,9 @@ Status S3FileSystem::CreateDir(const std::string& s, bool recursive) {
     // Check parent dir exists
     if (path.has_parent()) {
       S3Path parent_path = path.parent();
-      bool exists;
-      RETURN_NOT_OK(impl_->IsNonEmptyDirectory(parent_path, &exists));
+      ARROW_ASSIGN_OR_RAISE(bool exists, impl_->IsNonEmptyDirectory(parent_path));
       if (!exists) {
-        RETURN_NOT_OK(impl_->IsEmptyDirectory(parent_path, &exists));
+        ARROW_ASSIGN_OR_RAISE(exists, impl_->IsEmptyDirectory(parent_path));
       }
       if (!exists) {
         return Status::IOError("Cannot create directory '", path.full_path,
@@ -2215,7 +2370,7 @@ Status S3FileSystem::DeleteDir(const std::string& s) {
   if (path.empty()) {
     return Status::NotImplemented("Cannot delete all S3 buckets");
   }
-  RETURN_NOT_OK(impl_->DeleteDirContents(path.bucket, path.key));
+  RETURN_NOT_OK(impl_->DeleteDirContentsAsync(path.bucket, path.key).status());
   if (path.key.empty()) {
     // Delete bucket
     S3Model::DeleteBucketRequest req;
@@ -2232,14 +2387,20 @@ Status S3FileSystem::DeleteDir(const std::string& s) {
 }
 
 Status S3FileSystem::DeleteDirContents(const std::string& s) {
+  return DeleteDirContentsAsync(s).status();
+}
+
+Future<> S3FileSystem::DeleteDirContentsAsync(const std::string& s) {
   ARROW_ASSIGN_OR_RAISE(auto path, S3Path::FromString(s));
 
   if (path.empty()) {
     return Status::NotImplemented("Cannot delete all S3 buckets");
   }
-  RETURN_NOT_OK(impl_->DeleteDirContents(path.bucket, path.key));
-  // Directory may be implicitly deleted, recreate it
-  return impl_->EnsureDirectoryExists(path);
+  auto self = impl_;
+  return impl_->DeleteDirContentsAsync(path.bucket, path.key).Then([path, self]() {
+    // Directory may be implicitly deleted, recreate it
+    return self->EnsureDirectoryExists(path);
+  });
 }
 
 Status S3FileSystem::DeleteRootDirContents() {
@@ -2346,7 +2507,12 @@ Result<std::shared_ptr<io::OutputStream>> S3FileSystem::OpenAppendStream(
 // Top-level utility functions
 //
 
-Result<std::string> ResolveBucketRegion(const std::string& bucket) {
+Result<std::string> ResolveS3BucketRegion(const std::string& bucket) {
+  if (bucket.empty() || bucket.find_first_of(kSep) != bucket.npos ||
+      internal::IsLikelyUri(bucket)) {
+    return Status::Invalid("Not a valid bucket name: '", bucket, "'");
+  }
+
   ARROW_ASSIGN_OR_RAISE(auto resolver, RegionResolver::DefaultInstance());
   return resolver->ResolveRegion(bucket);
 }
